
function DamageMeters_Uninitialize()
	DamageMeters_UnregisterCommands();
end

function DamageMeters_Initialize()
	-- Initialize debug timers.
	for timer = 1, DMPROF_COUNT do
		DamageMeters_debugTimers[timer] = {};
		DamageMeters_debugTimers[timer].time = 0;
		DamageMeters_debugTimers[timer].count = 0;
		DamageMeters_debugTimers[timer].peak = 0;
	end

	DamageMetersFrame:RegisterEvent("VARIABLES_LOADED");
	
	-- Define DamageMeters_Quantity_MAX.
	-- ALL QUANTITIES MUST BE DEFINED BEFORE THIS POINT.
	DamageMeters_Quantity_MAX = table.getn(DM_QUANTDEFS);
	DMPrintD("DamageMeters_Quantity_MAX = "..DamageMeters_Quantity_MAX);

	-- Inititalize quantity color codes.
	for quant = 1, DamageMeters_Quantity_MAX do
		local color = DM_QUANTDEFS[quant].defaultColor;
		local code = string.format("|cFF%02X%02X%02X",
			floor(color[1] * 255.0),
			floor(color[2] * 255.0),
			floor(color[3] * 255.0));
		DamageMeters_quantityColorCodeDefault[quant] = code;
	end

	DamageMetersFrame:Show();
	if (not DamageMetersFrame:IsUserPlaced()) then
		DMPrintD("Not user placed: resetting pos.");
		DamageMeters_ResetPos();
	end

	-- Build arrays for easy reference of Bars and BarText, and initialize those elements.
	local name = DamageMetersFrame:GetName();
	local i;
	for i = 1,DamageMeters_BARCOUNT_MAX do
		DamageMeters_bars[i] = getglobal(name.."Bar"..i);
		DamageMeters_bars[i]:SetID(i);
		DamageMeters_bars[i]:Hide();
		DamageMeters_text[i] = getglobal(name.."Text"..i);
		SetTextStatusBarText(DamageMeters_bars[i], DamageMeters_text[i]);
		-- Force text on always.
		ShowTextStatusBarText(DamageMeters_bars[i]);
	end

	-- Initialize the sync bars.
	DamageMeters_sendMsgQueueBar = getglobal("DamageMetersFrame_SendMsgQueueBar");
	DamageMeters_sendMsgQueueBarText = getglobal("DamageMetersFrame_SendMsgQueueBarText");
	SetTextStatusBarText(DamageMeters_sendMsgQueueBar, DamageMeters_sendMsgQueueBarText);
	DamageMeters_sendMsgQueueBar:SetMinMaxValues(0, 100);
	DamageMeters_sendMsgQueueBar:SetValue(100);
	DamageMeters_sendMsgQueueBar:SetStatusBarColor(1.00, 0.0, 0.00);
	-- Force text on always.
	ShowTextStatusBarText(DamageMeters_sendMsgQueueBar);
	DamageMeters_sendMsgQueueBarText:SetPoint("CENTER", DamageMeters_sendMsgQueueBar:GetName(), "CENTER", 0, 0);
	DamageMeters_sendMsgQueueBarText:SetText("");
	DamageMeters_sendMsgQueueBar:Hide();

	DamageMeters_processMsgQueueBar = getglobal("DamageMetersFrame_ProcessMsgQueueBar");
	DamageMeters_processMsgQueueBarText = getglobal("DamageMetersFrame_ProcessMsgQueueBarText");
	SetTextStatusBarText(DamageMeters_processMsgQueueBar, DamageMeters_processMsgQueueBarText);
	DamageMeters_processMsgQueueBar:SetMinMaxValues(0, 250);
	DamageMeters_processMsgQueueBar:SetValue(0);
	DamageMeters_processMsgQueueBar:SetStatusBarColor(1.00, 0.0, 0.00);
	-- Force text on always.
	ShowTextStatusBarText(DamageMeters_processMsgQueueBar);
	DamageMeters_processMsgQueueBarText:SetPoint("CENTER", DamageMeters_processMsgQueueBar:GetName(), "CENTER", 0, 0);
	DamageMeters_processMsgQueueBarText:SetText("");
	DamageMeters_processMsgQueueBar:Hide();

	DamageMeters_RegisterCommands();

	-- Tell the frame to rebuild itself.
	DamageMeters_UpdateVisibility();
	DamageMeters_frameNeedsToBeGenerated = true;

	DamageMeters_lastUpdateTime = GetTime();
	DamageMeters_lastProcessQueueTime = DamageMeters_lastUpdateTime;
	DamageMeters_lastEventTime = GetTime();
end

function DamageMeters_On()
	DamageMeters_Initialize();

	-- If the saved data is of a different version clear the table.
	if (DamageMeters_loadedDataVersion ~= DamageMeters_VERSION) then
		DMPrintD("Saved data version mismatch: loaded = "..DamageMeters_loadedDataVersion..", current = "..DamageMeters_VERSION);
		DamageMeters_InitTables();
		
		DamageMeters_pluginDMITable = {};

		if (DamageMeters_loadedDataVersion < 4500) then
			DMPrintD("Restoring default colors.");
			DamageMeters_SetDefaultColors();
		end

		DamageMeters_loadedDataVersion = DamageMeters_VERSION;
	end

	-- Reset the ages, as age values from old sessions are pointless.
	-- Reset the combat dmg too.
	local index, value;
	local now = GetTime();
	for tableIndex = 1, DMT_MAX do
		for index, value in DamageMeters_tables[tableIndex] do
			value.lastTime = now;
			for quantIx = 1, DMI_MAX do
				value.dmiData[quantIx].lastQuantTime = now;
			end
		end
	end

	DamageMeters_Reset();

	DamageMeters_textStateStartTime = GetTime();
	DamageMeters_currentQuantStartTime = DamageMeters_textStateStartTime;

	DamageMeters_Show();

	-- Intro text.
	-- Removed by Sharak@NGAcn.com
	-- DamageMeters_ShowVersion();
	if (DamageMeters_flags[DMFLAG_accumulateToMemory]) then
		DMPrint(DM_MSG_ACCUMULATING);
	end
	if (DamageMeters_debugEnabled) then
		DMPrintD("DamageMeters: Debug enabled.");
	end
	--if (not DamageMeters_takeOnlyPlayerData) then
	--	DMPrint(DM_MSG_PLAYERONLYEVENTDATAOFF);
	--end
	
	DMPrintD("Loading DM into mode "..DamageMeters_viewMode..", count = "..DamageMeters_barCount..", saved = "..DamageMeters_savedBarCount);


	-- This is a hack--if the frame is initially hidden the stupid dropdown 
	-- menu doesn't work right and needs to be opened twice to be seen the first
	-- time.
	DMReportFrame:Hide();

	-- Make sure this is called last.
	DamageMeters_OnLoadComplete();

	-- This needs to know that DamageMeters_quantity is valid.
	DamageMeters_SetQuantity(DamageMeters_quantity, true);

	DamageMetersHiddenFrame:Show();

	DamageMeters_RegisterEvents();

	DamageMeters_Enabled = true;
end

function DamageMeters_Off()
	-- Hide approriate UI frames
	DMReportFrame:Hide();
	
	-- Hide DM panel
	DamageMetersFrame:Hide();
	DamageMetersFrame_TitleButton:Hide();
	DamageMetersFrame_TotalButton:Hide();

	DamageMeters_Uninitialize();

	DamageMeters_UnregisterEvents();

	DamageMeters_Enabled = false;
end

function DamageMeters_UnregisterCommands()
	-- Console commands.
	SlashCmdList["DAMAGEMETERSHELP"] = nil;
	SLASH_DAMAGEMETERSHELP1 = nil;
	SlashCmdList["DAMAGEMETERSCMD"] = nil;
	SLASH_DAMAGEMETERSCMD1 = nil;
	SlashCmdList["DAMAGEMETERSSHOW"] = nil;
	SLASH_DAMAGEMETERSSHOW1 = nil;
	SlashCmdList["DAMAGEMETERSHIDE"] = nil;
	SLASH_DAMAGEMETERSHIDE1 = nil;
	SlashCmdList["DAMAGEMETERSCLEAR"] = nil;
	SLASH_DAMAGEMETERSCLEAR1 = nil;
	SlashCmdList["DAMAGEMETERSREPORT"] = nil;
	SLASH_DAMAGEMETERSREPORT1 = nil;
	SlashCmdList["DAMAGEMETERSSORT"] = nil;
	SLASH_DAMAGEMETERSSORT1 = nil;
	SlashCmdList["DAMAGEMETERSCOUNT"] = nil;
	SLASH_DAMAGEMETERSCOUNT1 = nil;
	SlashCmdList["DAMAGEMETERSSAVE"] = nil;
	SLASH_DAMAGEMETERSSAVE1 = nil;
	SlashCmdList["DAMAGEMETERSRESTORE"] = nil;
	SLASH_DAMAGEMETERSRESTORE1 = nil;
	SlashCmdList["DAMAGEMETERSMERGE"] = nil;
	SLASH_DAMAGEMETERSMERGE1 = nil;
	SlashCmdList["DAMAGEMETERSSWAP"] = nil;
	SLASH_DAMAGEMETERSSWAP1 = nil;
	SlashCmdList["DAMAGEMETERSMEMCLEAR"] = nil;
	SLASH_DAMAGEMETERSMEMCLEAR1 = nil;
	SlashCmdList["DAMAGEMETERSRESETPOS"] = nil;
	SLASH_DAMAGEMETERSRESETPOS1 = nil;
	SlashCmdList["DAMAGEMETERSSHOWTEXT"] = nil;
	SLASH_DAMAGEMETERSSHOWTEXT1 = nil;
	SlashCmdList["DAMAGEMETERSCOLORSCHEME"] = nil;
	SLASH_DAMAGEMETERSCOLORSCHEME1 = nil;
	SlashCmdList["DAMAGEMETERSQUANTITY"] = nil;
	SLASH_DAMAGEMETERSQUANTITY1 = nil;
	SlashCmdList["DAMAGEMETERSVISINPARTY"] = nil;
	SLASH_DAMAGEMETERSVISINPARTY1 = nil;
	SlashCmdList["DAMAGEMETERSAUTOCOUNT"] = nil;
	SLASH_DAMAGEMETERSAUTOCOUNT1 = nil;
	SlashCmdList["DAMAGEMETERSLISTBANNED"] = nil;
	SLASH_DAMAGEMETERSLISTBANNED1 = nil;
	SlashCmdList["DAMAGEMETERSCLEARBANNED"] = nil;
	SLASH_DAMAGEMETERSCLEARBANNED1 = nil;
	SlashCmdList["DAMAGEMETERSSYNC"] = nil;
	SLASH_DAMAGEMETERSSYNC1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCCHAN"] = nil;
	SLASH_DAMAGEMETERSSYNCCHAN1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCLEAVE"] = nil;
	SLASH_DAMAGEMETERSSYNCLEAVE1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCSEND"] = nil;
	SLASH_DAMAGEMETERSSYNCSEND1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCREQUEST"] = nil;
	SLASH_DAMAGEMETERSSYNCREQUEST1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCCLEAR"] = nil;
	SLASH_DAMAGEMETERSSYNCCLEAR1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCMSG"] = nil;
	SLASH_DAMAGEMETERSSYNCMSG1 = nil;
	SLASH_DAMAGEMETERSSYNCMSG2 = nil;
	SlashCmdList["DAMAGEMETERSSYNCBROADCASTCHAN"] = nil;
	SLASH_DAMAGEMETERSSYNCBROADCASTCHAN1 = nil;
	SLASH_DAMAGEMETERSSYNCBROADCASTCHAN2 = nil;
	SlashCmdList["DAMAGEMETERSSYNCPING"] = nil;
	SLASH_DAMAGEMETERSSYNCPING1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCPAUSE"] = nil;
	SLASH_DAMAGEMETERSSYNCPAUSE1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCUNPAUSE"] = nil;
	SLASH_DAMAGEMETERSSYNCUNPAUSE1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCREADY"] = nil;
	SLASH_DAMAGEMETERSSYNCREADY1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCKICK"] = nil;
	SLASH_DAMAGEMETERSSYNCKICK1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCLABEL"] = nil;
	SLASH_DAMAGEMETERSSYNCLABEL1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCSTART"] = nil;
	SLASH_DAMAGEMETERSSYNCSTART1 = nil;
	SlashCmdList["DAMAGEMETERSSYNCHALT"] = nil;
	SLASH_DAMAGEMETERSSYNCHALT1 = nil;
	--SlashCmdList["DAMAGEMETERSSYNCBOSSSTART"] = DamageMeters_SyncBossStart;
	--SLASH_DAMAGEMETERSSYNCBOSSSTART1 = "/dmsyncbossstart";
	--SlashCmdList["DAMAGEMETERSSYNCBOSSEND"] = DamageMeters_SyncBossEnd;
	--SLASH_DAMAGEMETERSSYNCBOSSEND1 = "/dmsyncbossend";

	-- Undocumented atm.
	SlashCmdList["DAMAGEMETERSSYNCEMOTE"] = nil;
	SLASH_DAMAGEMETERSSYNCEMOTE1 = nil;
	SlashCmdList["DAMAGEMETERSRPS"] = nil;
	SLASH_DAMAGEMETERSRPS1 = nil;
	SlashCmdList["DAMAGEMETERSRPSR"] = nil;
	SLASH_DAMAGEMETERSRPSR1 = nil;

	SlashCmdList["DAMAGEMETERSPOPULATE"] = nil;
	SLASH_DAMAGEMETERSPOPULATE1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLELOCK"] = nil;
	SLASH_DAMAGEMETERSTOGGLELOCK1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLEPAUSE"] = nil;
	SLASH_DAMAGEMETERSTOGGLEPAUSE1 = nil;
	SlashCmdList["DAMAGEMETERSSETREADY"] = nil;
	SLASH_DAMAGEMETERSSETREADY1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLELOCKPOS"] = nil;
	SLASH_DAMAGEMETERSTOGGLELOCKPOS1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLEGROUPONLY"] = nil;
	SLASH_DAMAGEMETERSTOGGLEGROUPONLY1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLEADDPETTOPLAYER"] = nil;
	SLASH_DAMAGEMETERSTOGGLEADDPETTOPLAYER1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLERESETWHENCOMBATSTARTS"] = nil;
	SLASH_DAMAGEMETERSTOGGLERESETWHENCOMBATSTARTS1 = nil;
	SlashCmdList["DAMAGEMETERSVERSION"] = nil;
	SLASH_DAMAGEMETERSVERSION1 = nil;
	SLASH_DAMAGEMETERSVERSION2 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLETOTAL"] = nil;
	SLASH_DAMAGEMETERSTOGGLETOTAL1 = nil;
	SlashCmdList["DAMAGEMETERSTOGGLESHOWMAX"] = nil;
	SLASH_DAMAGEMETERSTOGGLESHOWMAX1 = nil;

	-- Commands for testing.
	-- ["reset"] = "/dmreset - (For Testing) Forces a re-layout of the visual elements.",
	SlashCmdList["DAMAGEMETERSRESET"] = nil;
	SLASH_DAMAGEMETERSRESET1 = nil;
	-- ["test"] = "/dmtest [#] - (For Testing) Adds # test entries to the list.  If no number specified, adds one entry for each visible bar.",
	SlashCmdList["DAMAGEMETERSTEST"] = nil;
	SLASH_DAMAGEMETERSTEST1 = nil;
	-- ["add"] = "/dmadd name - (For Testing) Simulates player 'name' doing 1 damage.",
	SlashCmdList["DAMAGEMETERSADD"] = nil;
	SLASH_DAMAGEMETERSADD1 = nil;
	-- ["dumptable"] = "/dmdumptable - (For Testing) Dumps the entire internal data table."
	SlashCmdList["DAMAGEMETERSDUMPTABLE"] = nil;
	SLASH_DAMAGEMETERSDUMPTABLE1 = nil;
	SlashCmdList["DAMAGEMETERSDEBUGPRINT"] = nil;
	SLASH_DAMAGEMETERSDEBUGPRINT1 = nil;
	SlashCmdList["DAMAGEMETERSDUMPMSG"] = nil;
	SLASH_DAMAGEMETERSDUMPMSG1 = nil;
	SlashCmdList["DAMAGEMETERSCONSOLEPRINT"] = nil;
	SLASH_DAMAGEMETERSCONSOLEPRINT1 = nil;
	SlashCmdList["DAMAGEMETERSCONSOLEPRINTTABLE"] = nil;
	SLASH_DAMAGEMETERSCONSOLEPRINTTABLE1 = nil;
end

function DamageMeters_RegisterCommands()
	-- Console commands.
	SlashCmdList["DAMAGEMETERSHELP"] = DamageMeters_Help;
	SLASH_DAMAGEMETERSHELP1 = "/dmhelp";
	SlashCmdList["DAMAGEMETERSCMD"] = DamageMeters_ListCommands;
	SLASH_DAMAGEMETERSCMD1 = "/dmcmd";
	SlashCmdList["DAMAGEMETERSSHOW"] = DamageMeters_ToggleShow;
	SLASH_DAMAGEMETERSSHOW1 = "/dmshow";
	SlashCmdList["DAMAGEMETERSHIDE"] = DamageMeters_Hide;
	SLASH_DAMAGEMETERSHIDE1 = "/dmhide";
	SlashCmdList["DAMAGEMETERSCLEAR"] = DamageMeters_Clear;
	SLASH_DAMAGEMETERSCLEAR1 = "/dmclear";
	SlashCmdList["DAMAGEMETERSREPORT"] = DamageMeters_Report;
	SLASH_DAMAGEMETERSREPORT1 = "/dmreport";
	SlashCmdList["DAMAGEMETERSSORT"] = DamageMeters_SetSort;
	SLASH_DAMAGEMETERSSORT1 = "/dmsort";
	SlashCmdList["DAMAGEMETERSCOUNT"] = DamageMeters_SetCount;
	SLASH_DAMAGEMETERSCOUNT1 = "/dmcount";
	SlashCmdList["DAMAGEMETERSSAVE"] = DamageMeters_Save;
	SLASH_DAMAGEMETERSSAVE1 = "/dmsave";
	SlashCmdList["DAMAGEMETERSRESTORE"] = DamageMeters_Restore;
	SLASH_DAMAGEMETERSRESTORE1 = "/dmrestore";
	SlashCmdList["DAMAGEMETERSMERGE"] = DamageMeters_Merge;
	SLASH_DAMAGEMETERSMERGE1 = "/dmmerge";
	SlashCmdList["DAMAGEMETERSSWAP"] = DamageMeters_Swap;
	SLASH_DAMAGEMETERSSWAP1 = "/dmswap";
	SlashCmdList["DAMAGEMETERSMEMCLEAR"] = DamageMeters_MemClear;
	SLASH_DAMAGEMETERSMEMCLEAR1 = "/dmmemclear";
	SlashCmdList["DAMAGEMETERSRESETPOS"] = DamageMeters_ResetPos;
	SLASH_DAMAGEMETERSRESETPOS1 = "/dmresetpos";
	SlashCmdList["DAMAGEMETERSSHOWTEXT"] = DamageMeters_SetTextOptions;
	SLASH_DAMAGEMETERSSHOWTEXT1 = "/dmtext";
	SlashCmdList["DAMAGEMETERSCOLORSCHEME"] = DamageMeters_SetColorScheme;
	SLASH_DAMAGEMETERSCOLORSCHEME1 = "/dmcolor";
	SlashCmdList["DAMAGEMETERSQUANTITY"] = DamageMeters_SetQuantity;
	SLASH_DAMAGEMETERSQUANTITY1 = "/dmquant";
	SlashCmdList["DAMAGEMETERSVISINPARTY"] = DamageMeters_SetVisibleInParty;
	SLASH_DAMAGEMETERSVISINPARTY1 = "/dmvisinparty";
	SlashCmdList["DAMAGEMETERSAUTOCOUNT"] = DamageMeters_SetAutoCount;
	SLASH_DAMAGEMETERSAUTOCOUNT1 = "/dmautocount";
	SlashCmdList["DAMAGEMETERSLISTBANNED"] = DamageMeters_ListBanned;
	SLASH_DAMAGEMETERSLISTBANNED1 = "/dmlistbanned";
	SlashCmdList["DAMAGEMETERSCLEARBANNED"] = DamageMeters_ClearBanned;
	SLASH_DAMAGEMETERSCLEARBANNED1 = "/dmclearbanned";
	SlashCmdList["DAMAGEMETERSSYNC"] = DamageMeters_Sync;
	SLASH_DAMAGEMETERSSYNC1 = "/dmsync";
	SlashCmdList["DAMAGEMETERSSYNCSEND"] = DamageMeters_SyncReport;
	SLASH_DAMAGEMETERSSYNCSEND1 = "/dmsyncsend";
	SlashCmdList["DAMAGEMETERSSYNCREQUEST"] = DamageMeters_SyncRequest;
	SLASH_DAMAGEMETERSSYNCREQUEST1 = "/dmsyncrequest";
	SlashCmdList["DAMAGEMETERSSYNCCLEAR"] = DamageMeters_SyncClear;
	SLASH_DAMAGEMETERSSYNCCLEAR1 = "/dmsyncclear";
	SlashCmdList["DAMAGEMETERSSYNCMSG"] = DamageMeters_SendSyncMsg;
	SLASH_DAMAGEMETERSSYNCMSG1 = "/dmsyncmsg";
	SLASH_DAMAGEMETERSSYNCMSG2 = "/dmm";
	SlashCmdList["DAMAGEMETERSSYNCPING"] = DamageMeters_SyncPingRequest;
	SLASH_DAMAGEMETERSSYNCPING1 = "/dmsyncping";
	SlashCmdList["DAMAGEMETERSSYNCPAUSE"] = DamageMeters_SyncPause;
	SLASH_DAMAGEMETERSSYNCPAUSE1 = "/dmsyncpause";
	SlashCmdList["DAMAGEMETERSSYNCUNPAUSE"] = DamageMeters_SyncUnpause;
	SLASH_DAMAGEMETERSSYNCUNPAUSE1 = "/dmsyncunpause";
	SlashCmdList["DAMAGEMETERSSYNCREADY"] = DamageMeters_SyncReady;
	SLASH_DAMAGEMETERSSYNCREADY1 = "/dmsyncready";
	SlashCmdList["DAMAGEMETERSSYNCLABEL"] = DamageMeters_SyncLabel;
	SLASH_DAMAGEMETERSSYNCLABEL1 = "/dmsynclabel";
	SlashCmdList["DAMAGEMETERSSYNCSTART"] = DamageMeters_SyncStart;
	SLASH_DAMAGEMETERSSYNCSTART1 = "/dmsyncstart";
	SlashCmdList["DAMAGEMETERSSYNCHALT"] = DamageMeters_SyncHalt;
	SLASH_DAMAGEMETERSSYNCHALT1 = "/dmsynchalt";
	--SlashCmdList["DAMAGEMETERSSYNCBOSSSTART"] = DamageMeters_SyncBossStart;
	--SLASH_DAMAGEMETERSSYNCBOSSSTART1 = "/dmsyncbossstart";
	--SlashCmdList["DAMAGEMETERSSYNCBOSSEND"] = DamageMeters_SyncBossEnd;
	--SLASH_DAMAGEMETERSSYNCBOSSEND1 = "/dmsyncbossend";

	-- Undocumented atm.
	SlashCmdList["DAMAGEMETERSSYNCEMOTE"] = DamageMeters_SyncEmote;
	SLASH_DAMAGEMETERSSYNCEMOTE1 = "/dme";
	SlashCmdList["DAMAGEMETERSRPS"] = DamageMeters_RPSChallenge;
	SLASH_DAMAGEMETERSRPS1 = "/dmrps";
	SlashCmdList["DAMAGEMETERSRPSR"] = DamageMeters_RPSResponse;
	SLASH_DAMAGEMETERSRPSR1 = "/dmrpsr";

	SlashCmdList["DAMAGEMETERSPOPULATE"] = DamageMeters_Populate;
	SLASH_DAMAGEMETERSPOPULATE1 = "/dmpop";
	SlashCmdList["DAMAGEMETERSTOGGLELOCK"] = DamageMeters_ToggleLock;
	SLASH_DAMAGEMETERSTOGGLELOCK1 = "/dmlock";
	SlashCmdList["DAMAGEMETERSTOGGLEPAUSE"] = DamageMeters_TogglePause;
	SLASH_DAMAGEMETERSTOGGLEPAUSE1 = "/dmpause";
	SlashCmdList["DAMAGEMETERSSETREADY"] = DamageMeters_SetReady;
	SLASH_DAMAGEMETERSSETREADY1 = "/dmready";
	SlashCmdList["DAMAGEMETERSTOGGLELOCKPOS"] = DamageMeters_ToggleLockPos;
	SLASH_DAMAGEMETERSTOGGLELOCKPOS1 = "/dmlockpos";
	SlashCmdList["DAMAGEMETERSTOGGLEGROUPONLY"] = DamageMeters_ToggleGroupMembersOnly;
	SLASH_DAMAGEMETERSTOGGLEGROUPONLY1 = "/dmgrouponly";
	SlashCmdList["DAMAGEMETERSTOGGLEADDPETTOPLAYER"] = DamageMeters_ToggleAddPetToPlayer;
	SLASH_DAMAGEMETERSTOGGLEADDPETTOPLAYER1 = "/dmaddpettoplayer";
	SlashCmdList["DAMAGEMETERSTOGGLERESETWHENCOMBATSTARTS"] = DamageMeters_ToggleResetWhenCombatStarts;
	SLASH_DAMAGEMETERSTOGGLERESETWHENCOMBATSTARTS1 = "/dmresetoncombat";
	SlashCmdList["DAMAGEMETERSVERSION"] = DamageMeters_ShowVersion;
	SLASH_DAMAGEMETERSVERSION1 = "/dmversion";
	SLASH_DAMAGEMETERSVERSION2 = "/dmver";
	SlashCmdList["DAMAGEMETERSTOGGLETOTAL"] = DamageMeters_ToggleTotal;
	SLASH_DAMAGEMETERSTOGGLETOTAL1 = "/dmtotal";
	SlashCmdList["DAMAGEMETERSTOGGLESHOWMAX"] = DamageMeters_ToggleMaxBars;
	SLASH_DAMAGEMETERSTOGGLESHOWMAX1 = "/dmshowmax";

	-- Commands for testing.
	-- ["reset"] = "/dmreset - (For Testing) Forces a re-layout of the visual elements.",
	SlashCmdList["DAMAGEMETERSRESET"] = DamageMeters_Reset;
	SLASH_DAMAGEMETERSRESET1 = "/dmreset";
	-- ["test"] = "/dmtest [#] - (For Testing) Adds # test entries to the list.  If no number specified, adds one entry for each visible bar.",
	SlashCmdList["DAMAGEMETERSTEST"] = DamageMeters_Test;
	SLASH_DAMAGEMETERSTEST1 = "/dmtest";
	-- ["add"] = "/dmadd name - (For Testing) Simulates player 'name' doing 1 damage.",
	SlashCmdList["DAMAGEMETERSADD"] = DamageMeters_Add;
	SLASH_DAMAGEMETERSADD1 = "/dmadd";
	-- ["dumptable"] = "/dmdumptable - (For Testing) Dumps the entire internal data table."
	SlashCmdList["DAMAGEMETERSDUMPTABLE"] = DamageMeters_DumpTable;
	SLASH_DAMAGEMETERSDUMPTABLE1 = "/dmdumptable";
	SlashCmdList["DAMAGEMETERSDEBUGPRINT"] = DM_ToggleDMPrintD;
	SLASH_DAMAGEMETERSDEBUGPRINT1 = "/dmdebug";
	SlashCmdList["DAMAGEMETERSDUMPMSG"] = DM_DumpMsg;
	SLASH_DAMAGEMETERSDUMPMSG1 = "/dmdumpmsg";
	SlashCmdList["DAMAGEMETERSCONSOLEPRINT"] = DM_ConsolePrint;
	SLASH_DAMAGEMETERSCONSOLEPRINT1 = "/dmp";
	SlashCmdList["DAMAGEMETERSCONSOLEPRINTTABLE"] = DM_ConsolePrintTable;
	SLASH_DAMAGEMETERSCONSOLEPRINTTABLE1 = "/dmpt";
end